/*************************************************
Title: Hierarchical Bayes Approach for Subgroup Analysis      
Authors: Y.-Y. Hsu, J. Zalkikar and R. C. Tiwari
Submitted to: Statistical Methods in Medical Research
Date: May 2017
Appendix: SAS Code for Table 1
**************************************************/

data itrdata ;
infile 'SimulatedData.csv' delimiter = ',' MISSOVER DSD lrecl=32767 firstobs=2 ;
informat myidxsubj best32. ;
informat myidxgrp best32. ;
informat seq best32. ;
informat period best32. ;
informat visit best32. ;
informat trt best32. ;
informat Y best32. ;
input myidxsubj myidxgrp seq period visit trt Y ;
run;

*begin:RMLE model (3) => Figure-5, Table-1 ****************************;
proc mixed data=itrdata METHOD=REML;
class myidxsubj period myidxgrp;
model Y = myidxgrp trt/ ddfm=KENWARDROGER;
random trt*myidxgrp ;
random period / subject=myidxsubj(myidxgrp);
Estimate "deltag0" trt 1 / cl;
Estimate "deltag1" trt 1 | trt*myidxgrp 1 0 0 0 / cl;
Estimate "deltag2" trt 1 | trt*myidxgrp 0 1 0 0 / cl;
Estimate "deltag3" trt 1 | trt*myidxgrp 0 0 1 0 / cl;
Estimate "deltag4" trt 1 | trt*myidxgrp 0 0 0 1 / cl;
Estimate "deltagw" trt 1 | trt*myidxgrp 0.06250 0.15625 0.31250 0.46875/cl;
run;

*begin:RMLE model (2) => Figure-5 *******************************;
proc mixed data=itrdata METHOD=REML;
class myidxsubj period visit myidxgrp;
model Y = myidxgrp trt trt*myidxgrp/ ddfm=KENWARDROGER;
random period / subject=myidxsubj(myidxgrp);
Estimate "deltag1" trt 1 trt*myidxgrp 1 0 0 0 / cl;
Estimate "deltag2" trt 1 trt*myidxgrp 0 1 0 0 / cl;
Estimate "deltag3" trt 1 trt*myidxgrp 0 0 1 0 / cl;
Estimate "deltag4" trt 1 trt*myidxgrp 0 0 0 1 / cl;
Estimate "deltagw" trt 1 trt*myidxgrp 0.06250 0.15625 0.31250 0.46875/cl;
run;

%let parnmcmc = 10000;
%let parnthin = 10;
%let parnhpdl = %eval(&parnmcmc * 5 / 100 );

*begin:HB model (3) => Table-1: IG ******************************;
proc mcmc data=itrdata seed=35753 nbi=10000 nmc=%eval(&parnmcmc*&parnthin) 
thin=&parnthin OUTPOST=post1
stats=all monitor=(mydelta asdres asdgpd asdper) DIC PLOTS=none diag=(GEWEKE ESS);

array alphag[4] ;
parms mydelta 0;
parms alphag: 0;
parms vgpd 1 ;
parms vper 1 ;
parms vres 1 ;

beginprior;
prior vres ~ igamma(0.001, scale=0.001);
prior vper ~ igamma(0.001, scale=0.001);
prior vgpd ~ igamma(0.001, scale=0.001);

asdres = sqrt(vres);
asdgpd = sqrt(vgpd);
asdper = sqrt(vper);

prior mydelta ~ normal(0, sd=100);
prior alpha: ~ normal(0, sd=100);
endprior;
random mydeltaj ~ normal(mydelta, sd=asdgpd) subject=myidxgrp monitor=(mydeltaj);
random myper ~ normal(0, sd=asdper) subject=myidxsubj;
mui = alphag[myidxgrp] + mydeltaj*Trt + myper;
model Y ~ normal(mui, sd = asdres);
run;

%macro prthpd( mdata, mvar );
data &mdata; set &mdata;
deltaw = (4*mydeltaj_1+ 10*mydeltaj_2 + 20*mydeltaj_3 + 30*mydeltaj_4)/64; 
mindelta = min( mydeltaj_1, mydeltaj_2, mydeltaj_3, mydeltaj_4);
run;
data mdata2; set &mdata; 
keep &mvar;
run;
proc sort data=mdata2 out=mdata2;
by &mvar;
run;
data mdata3;
merge mdata2( firstobs=1 obs=&parnhpdl rename=(&mvar=deltawHPDL)) 
mdata2( firstobs=%eval(&parnmcmc-&parnhpdl+1) obs=&parnmcmc rename=(&mvar=deltawHPDU));
HPDlength = deltawHPDU-deltawHPDL;
run;
proc sort data=mdata3 out=mdata3;
by HPDlength ;
run;
proc print data=mdata3(obs=1);
run;
proc means data=&mdata mean std;
var deltaw mindelta;
run;
ods graphics on;
%tadplot(data=&mdata, var= mydeltaj_1 mydeltaj_2 mydeltaj_3 mydeltaj_4
deltaw mydelta asdper asdres asdgpd );
ods graphics off;
%mend;

%prthpd( post1, deltaw );

*begin:HB model (3) => Table-1: Uniform *****************************************;
proc mcmc data=itrdata seed=35553 nbi=10000 nmc=%eval(&parnmcmc*&parnthin)
thin=&parnthin OUTPOST=post2
stats=all monitor=(mydelta asdres asdgpd asdper) DIC PLOTS=none diag=(GEWEKE ESS);
array alphag[4] ;
parms mydelta 0;
parms alphag: 0;
parms asdgpd 1 ;
parms vper 1 ;
parms vres 1 ;

beginprior;
prior vres ~ igamma(0.001, scale=0.001);
prior vper ~ igamma(0.001, scale=0.001);
prior asdgpd ~ uniform(0, 10);

asdres = sqrt(vres);
asdper = sqrt(vper);

prior mydelta ~ normal(0, sd=100);
prior alpha: ~ normal(0, sd=100);
endprior;
random mydeltaj ~ normal(mydelta, sd=asdgpd) subject=myidxgrp monitor=(mydeltaj);
random myper ~ normal(0, sd=asdper) subject=myidxsubj;
mui = alphag[myidxgrp] + mydeltaj*Trt + myper;
model Y ~ normal(mui, sd = asdres);
run;

%prthpd( post2, deltaw );

*begin:HB model (3) => Table-1: HC_1 **************************************;
proc mcmc data=itrdata seed=35353 nbi=10000 nmc=%eval(&parnmcmc*&parnthin) 
thin=&parnthin OUTPOST=post3
stats=all monitor=(mydelta asdres asdgpd asdper) DIC PLOTS=none diag=(GEWEKE ESS);
array alphag[4] ;
parms mydelta 0;
parms alphag: 0;
parms asdgpd 1 ;
parms vper 1 ;
parms vres 1 ;

beginprior;
prior vres ~ igamma(0.001, scale=0.001);
prior vper ~ igamma(0.001, scale=0.001);
prior asdgpd ~ cauchy(0, 0.15, lower=1e-12);

asdres = sqrt(vres);
asdper = sqrt(vper);

prior mydelta ~ normal(0, sd=100);
prior alpha: ~ normal(0, sd=100);
endprior;
random mydeltaj ~ normal(mydelta, sd=asdgpd) subject=myidxgrp monitor=(mydeltaj);
random myper ~ normal(0, sd=asdper) subject=myidxsubj;
mui = alphag[myidxgrp] + mydeltaj*Trt + myper;
model Y ~ normal(mui, sd = asdres);
ods output PostSummaries=postsum PostIntervals=postcl DIC=bdic Geweke=dgeweke ESS=dess ; 
run;

%prthpd( post3, deltaw );

*begin:HB model (3) => Table-1: HC_2 *******************************************************;
proc mcmc data=itrdata seed=35053 nbi=10000 nmc=%eval(&parnmcmc*&parnthin) 
thin=&parnthin OUTPOST=post4
stats=all monitor=(mydelta asdres asdgpd asdper) DIC PLOTS=none diag=(GEWEKE ESS);
ods select PostSummaries PostIntervals DIC Geweke ESS ;

array alphag[4] ;
parms mydelta 0;
parms alphag: 0;
parms asdgpd 1 ;
parms vper 1 ;
parms vres 1 ;

beginprior;
prior vres ~ igamma(0.001, scale=0.001);
prior vper ~ igamma(0.001, scale=0.001);
prior asdgpd ~ cauchy(0, 0.6, lower=1e-12);

asdres = sqrt(vres);
asdper = sqrt(vper);

prior mydelta ~ normal(0, sd=100);
prior alpha: ~ normal(0, sd=100);
endprior;
random mydeltaj ~ normal(mydelta, sd=asdgpd) subject=myidxgrp monitor=(mydeltaj);
random myper ~ normal(0, sd=asdper) subject=myidxsubj;
mui = alphag[myidxgrp] + mydeltaj*Trt + myper;
model Y ~ normal(mui, sd = asdres);
ods output PostSummaries=postsum PostIntervals=postcl DIC=bdic Geweke=dgeweke ESS=dess ; 
run;

%prthpd( post4, deltaw );

